@@ -3,6 +3,7 @@ package Lemonldap::NG::Portal::2F::Register::WebAuthn;
 
 use strict;
 use Mouse;
+use Lemonldap::NG::Portal::Main::Constants 'PE_OK';
 use JSON qw(from_json to_json);
 use MIME::Base64 qw(encode_base64url decode_base64url);
 use Crypt::URandom;
@@ -52,6 +53,24 @@ has rpName => (
     }
 );
 
+use constant supportedActions => {
+    registrationchallenge => "_registrationchallenge",
+    registration          => "_registration",
+    verificationchallenge => "_verificationchallenge",
+    verification          => "_verification",
+    delete                => "delete",
+};
+
+sub initDisplay {
+    my ( $self, $req ) = @_;
+
+    $req->data->{customScript} .= <<"EOF";
+<script type="text/javascript" src="$self->{p}->{staticPrefix}/common/js/webauthn-json.browser-global.min.js"></script>
+<script type="text/javascript" src="$self->{p}->{staticPrefix}/common/js/webauthnregistration.min.js"></script>
+EOF
+
+}
+
 # Split content of webauthn2fAttestationTrust into an array ref of PEM certificates
 sub _build_trust_anchors {
     my ($self) = shift;
@@ -92,12 +111,14 @@ sub _generate_user_handle {
 }
 
 sub _registrationchallenge {
-    my ( $self, $req, $user ) = @_;
+    my ( $self, $req ) = @_;
+    my $user             = $req->userData->{ $self->conf->{whatToTrace} };
     my @alldevices       = $self->find2fDevicesByType( $req, $req->userData );
     my $challenge_base64 = encode_base64url( Crypt::URandom::urandom(32) );
 
     my $displayName      = $req->userData->{ $self->displayname_attr } || $user;
     my $userVerification = $self->conf->{webauthn2fUserVerification};
+    my $residentKey      = $self->conf->{webauthn2fResidentKey};
     my $attestation      = $self->conf->{webauthn2fAttestation} || "none";
     my $request          = {
         rp => {
@@ -113,8 +134,11 @@ sub _registrationchallenge {
         attestation            => $attestation,
         pubKeyCredParams       => [],
         authenticatorSelection => { (
-                $userVerification
-                ? ( userVerification => $userVerification )
+                $userVerification ? ( userVerification => $userVerification )
+                : ()
+            ),
+            (
+                $residentKey ? ( residentKey => $residentKey )
                 : ()
             )
         }
@@ -128,31 +152,32 @@ sub _registrationchallenge {
 
     $self->logger->debug(
         "WebAuthn registration parameters " . to_json($request) );
-    return $self->p->sendJSONresponse( $req,
+    return $self->successResponse( $req,
         { request => $request, state_id => $token } );
 }
 
 sub _registration {
-    my ( $self, $req, $user ) = @_;
+    my ( $self, $req ) = @_;
+    my $user = $req->userData->{ $self->conf->{whatToTrace} };
 
     # Recover creation parameters, including challenge
     my $state_id = $req->param('state_id');
     unless ($state_id) {
         $self->logger->error(
             $self->prefix . "2f: could not find state ID in response" );
-        return $self->p->sendError( $req, 'webAuthnRegisterFailed', 400 );
+        return $self->failResponse( $req, 'webAuthnRegisterFailed', 400 );
     }
     my $state_data;
     unless ( $state_data = $self->ott->getToken($state_id) ) {
         $self->logger->error( $self->prefix
               . "2f: expired or invalid state ID in response: $state_id" );
-        return $self->p->sendError( $req, 'PE82', 400 );
+        return $self->failResponse( $req, 'PE82', 400 );
     }
     my $registration_options = ( $state_data->{registration_options} );
     unless ($registration_options) {
         $self->logger->error( $self->prefix
               . '2f: registration options missing from state data' );
-        return $self->p->sendError( $req, 'webAuthnRegisterFailed', 400 );
+        return $self->failResponse( $req, 'webAuthnRegisterFailed', 400 );
     }
 
     # Data required for WebAuthn verification
@@ -163,7 +188,7 @@ sub _registration {
     unless ($credential_json) {
         $self->logger->error(
             $self->prefix . '2f: missing credential parameter' );
-        return $self->p->sendError( $req, 'webAuthnRegisterFailed', 400 );
+        return $self->failResponse( $req, 'webAuthnRegisterFailed', 400 );
     }
 
     my $validation = eval {
@@ -173,7 +198,7 @@ sub _registration {
     if ($@) {
         $self->logger->error(
             $self->prefix . "2f: Credential validation error: $@" );
-        return $self->p->sendError( $req, "webAuthnRegisterFailed", 400 );
+        return $self->failResponse( $req, "webAuthnRegisterFailed", 400 );
     }
 
     my $credential_id     = $validation->{credential_id};
@@ -190,41 +215,55 @@ sub _registration {
             ( $aaguid ? "AAGUID: $aaguid" : () ) )
     );
 
-    return $self->p->sendError( $req, 'webauthnAlreadyRegistered', 400 )
+    return $self->failResponse( $req, 'webauthnAlreadyRegistered', 400 )
       if $self->find2fDevicesByKey( $req, $req->userData, $self->type,
         "_credentialId", $credential_id );
 
     my $keyName =
       $self->checkNameSfa( $req, $self->type, $req->param('keyName') );
-    return $self->p->sendError( $req, 'badName', 200 ) unless $keyName;
+    return $self->failResponse( $req, 'badName', 200 ) unless $keyName;
 
-    if (
-        $self->add2fDevice(
-            $req,
-            $req->userData,
-            {
-                _credentialId        => $credential_id,
-                _credentialPublicKey => $credential_pubkey,
-                _signCount           => $signature_count,
-                ( $aaguid ? ( _aaguid => $aaguid ) : () ),
-                type  => $self->type,
-                name  => $keyName,
-                epoch => time(),
-            }
-        )
-      )
-    {
-        $self->markRegistered($req);
-        return $self->p->sendJSONresponse( $req, { result => 1 } );
+    my $is_resident = (
+        $registration_options->{authenticatorSelection}->{residentKey}
+          and
+          ( $registration_options->{authenticatorSelection}->{residentKey} eq
+            "required" )
+    );
+
+    my $serialized_transports =
+      $self->_serializeTransportsFromJsonResponse($credential_json);
+    my $res = $self->registerDevice(
+        $req,
+        $req->userData,
+        {
+            _credentialId        => $credential_id,
+            _credentialPublicKey => $credential_pubkey,
+            _signCount           => $signature_count,
+            (
+                $serialized_transports
+                ? ( _transports => $serialized_transports )
+                : ()
+            ),
+            ( $aaguid ? ( _aaguid => $aaguid ) : () ),
+            type  => $self->type,
+            name  => $keyName,
+            epoch => time(),
+            ( $is_resident ? ( resident => 1 ) : () ),
+        }
+    );
+
+    if ( $res == PE_OK ) {
+        return $self->successResponse( $req, { result => 1 } );
     }
     else {
         $self->logger->error( $self->prefix . '2f: unable to add device' );
-        return $self->p->sendError( $req, 'serverError' );
+        return $self->failResponse( $req, "PE$res" );
     }
 }
 
 sub _verificationchallenge {
-    my ( $self, $req, $user ) = @_;
+    my ( $self, $req ) = @_;
+    my $user = $req->userData->{ $self->conf->{whatToTrace} };
 
     $self->logger->debug( $self->prefix . '2f: verification challenge req' );
 
@@ -232,7 +271,7 @@ sub _verificationchallenge {
 
     unless ($request) {
         $self->logger->error( $self->prefix . '2f: no registered device' );
-        return $self->p->sendError( $req, 'webAuthnNoDevice', 500 );
+        return $self->failResponse( $req, 'webAuthnNoDevice', 500 );
     }
 
     # Request is persisted on the server
@@ -243,26 +282,27 @@ sub _verificationchallenge {
 
     $self->logger->debug(
         $self->prefix . "2f: authentication parameters: " . to_json($request) );
-    return $self->p->sendJSONresponse( $req,
+    return $self->successResponse( $req,
         { request => $request, state_id => $token } );
 }
 
 sub _verification {
-    my ( $self, $req, $user ) = @_;
+    my ( $self, $req ) = @_;
+    my $user = $req->userData->{ $self->conf->{whatToTrace} };
 
     my $credential_json = $req->param('credential');
 
     unless ($credential_json) {
         $self->logger->error(
             $self->prefix . '2f: missing credential parameter' );
-        return $self->p->sendError( $req, 'webAuthnFailed', 400 );
+        return $self->failResponse( $req, 'webAuthnFailed', 400 );
     }
 
     my $state_id = $req->param('state_id');
     unless ($state_id) {
         $self->logger->error( $self->prefix
               . "2f: could not find state ID in response ($credential_json)" );
-        return $self->p->sendError( $req, 'webAuthnFailed', 400 );
+        return $self->failResponse( $req, 'webAuthnFailed', 400 );
     }
 
     # Recover challenge
@@ -270,7 +310,7 @@ sub _verification {
     unless ( $state_data = $self->ott->getToken($state_id) ) {
         $self->logger->error( $self->prefix
               . "2f: expired or invalid state ID in response ($state_id)" );
-        return $self->p->sendError( $req, 'PE82', 400 );
+        return $self->failResponse( $req, 'PE82', 400 );
     }
 
     my $signature_options = ( $state_data->{authentication_options} );
@@ -281,70 +321,14 @@ sub _verification {
     if ($@) {
         $self->logger->error(
             $self->prefix . "2f: validation error for $user: $@" );
-        return $self->p->sendJSONresponse( $req, { result => 0 } );
+        return $self->successResponse( $req, { result => 0 } );
     }
 
     if ( $validation_result->{success} == 1 ) {
-        return $self->p->sendJSONresponse( $req, { result => 1 } );
+        return $self->successResponse( $req, { result => 1 } );
     }
     else {
-        return $self->p->sendJSONresponse( $req, { result => 0 } );
-    }
-}
-
-sub _delete {
-    my ( $self, $req, $user ) = @_;
-
-    # Check if unregistration is allowed
-    return $self->p->sendError( $req, 'notAuthorized', 400 )
-      unless $self->userCanRemove;
-
-    my $epoch = $req->param('epoch');
-    unless ($epoch) {
-        $self->logger->error(
-            $self->prefix . '2f: "epoch" parameter is missing' );
-        return $self->p->sendError( $req, '2FDeviceNotFound', 400 );
-    }
-
-    return $self->del2fDevice( $req, $req->userData, $self->type, $epoch )
-      ? $self->p->sendJSONresponse( $req, { result => 1 } )
-      : $self->p->sendError( $req, '2FDeviceNotFound', 400 );
-}
-
-# Main method
-sub run {
-    my ( $self, $req, $action ) = @_;
-    my $user = $req->userData->{ $self->conf->{whatToTrace} };
-
-    unless ($user) {
-        $self->logger->error(
-            'No ' . $self->conf->{whatToTrace} . ' found in user data' );
-        return $self->p->sendError( $req, 'serverError', 500 );
-    }
-
-    if ( $action eq 'registrationchallenge' ) {
-        return $self->_registrationchallenge( $req, $user );
-    }
-
-    elsif ( $action eq 'registration' ) {
-        return $self->_registration( $req, $user );
-    }
-
-    elsif ( $action eq 'verificationchallenge' ) {
-        return $self->_verificationchallenge( $req, $user );
-    }
-
-    elsif ( $action eq 'verification' ) {
-        return $self->_verification( $req, $user );
-    }
-
-    elsif ( $action eq 'delete' ) {
-        return $self->_delete( $req, $user );
-    }
-
-    else {
-        $self->logger->error( $self->prefix . "2f: unknown action -> $action" );
-        return $self->p->sendError( $req, 'unknownAction', 400 );
+        return $self->successResponse( $req, { result => 0 } );
     }
 }
 
