@@ -63,7 +63,9 @@ use Time::Fake;
 use URI::Escape;
 use MIME::Base64;
 use Lemonldap::NG::Common::FormEncode;
+use Lemonldap::NG::Common::Session 'id2storage';
 use Lemonldap::NG::Common::Util qw/getPSessionID/;
+use Carp                        qw/shortmess/;
 
 #use 5.10.0;
 
@@ -108,18 +110,9 @@ Convenience method that builds a url-encoded query string from a hash of argumen
 
 sub buildForm {
     my $fields = shift;
-    my $query  = join(
-        '&',
-        map {
-            "$_="
-              . (
-                $fields->{$_}
-                ? uri_escape( uri_unescape( $fields->{$_} ) )
-                : ''
-              )
-          }
-          keys(%$fields)
-    );
+    my $query  = join( '&',
+        map { "$_=" . ( $fields->{$_} ? uri_escape( $fields->{$_} ) : '' ) }
+          keys(%$fields) );
     return $query;
 }
 
@@ -188,7 +181,8 @@ sub getCache {
 }
 
 sub getSession {
-    my $id           = shift;
+    my $id = shift;
+    $id = $ENV{LLNG_HASHED_SESSION_STORE} ? id2storage($id) : $id;
     my @sessionsOpts = (
         storageModule        => "Apache::Session::File",
         storageModuleOptions => {
@@ -293,6 +287,20 @@ sub expectAutoPost {
     return @r;
 }
 
+sub getJsVars {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    my ($res) = @_;
+
+    my $initscripts =
+      getHtmlElement( $res, '//script[@type="application/init"]' );
+
+    my @parsed_initscripts =
+      map { from_json( $_->string_value ) } $initscripts->get_nodelist();
+
+    my %vars = map { %$_ } @parsed_initscripts;
+    return \%vars;
+}
+
 =head4 expectForm( $res, $hostRe, $uriRe, @requiredFields )
 
 Verify form in HTML result and return ( $host, $uri, $query, $method ):
@@ -421,6 +429,7 @@ sub expectSessionAttributes {
 sub getSessionAttributes {
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ( $app, $id ) = @_;
+    $id = $ENV{LLNG_HASHED_SESSION_STORE} ? id2storage($id) : $id;
     my $res;
     ok(
         $res = $app->_get("/sessions/global/$id"),
@@ -629,6 +638,17 @@ sub getHtmlElement {
     return $doc->findnodes($xpath);
 }
 
+sub expectXpath {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my ( $res, $xpath, $message ) = @_;
+    $message ||= "Found at least one result for $xpath";
+
+    ok( $res = getHtmlElement( $res, $xpath ), $message, );
+    count(1);
+    return $res;
+}
+
 =head4 getCookies($res)
 
 Returns an hash ref with names => values of cookies set by server.
@@ -702,38 +722,125 @@ sub tempdb {
 }
 
 my %handlerOR;
+my %handlerTSHV;
 
 =head4 register
 
-Registers a new LLNG instance
+Multi-handler system. This automatically takes care of loading the correct
+handler when processing a request.
+
 
 =cut
 
+our @currenthandler;
+
 sub register {
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     my ( $type, $constructor ) = @_;
     my $obj;
+
+    # Clear previous global handler data
     @Lemonldap::NG::Handler::Main::_onReload = ();
+    $Lemonldap::NG::Handler::Main::_tshv     = {
+        tsv             => {},
+        cfgNum          => 0,
+        cfgDate         => 0,
+        lastCheck       => 0,
+        checkTime       => 600,
+        confAcc         => {},
+        logger          => {},
+        userLogger      => {},
+        lmConf          => {},
+        localConfig     => {},
+        _auditLogger    => {},
+    };
     &Lemonldap::NG::Handler::Main::cfgNum( 0, 0 );
+
+    # Set currenthandler stack to the type being initialized
+    @currenthandler = $type;
+
     ok( $obj = $constructor->(), 'Register $type' );
+
+    # If the constructed object has an app (most cases)
+    # we wrap the app in a function that loads the correct handler context
+    # before processing the request, and unloads if afterwards
+    use Scalar::Util 'blessed';
+    if ( $obj and blessed($obj) and $obj->can('app') ) {
+        my $inner_app = $obj->app;
+
+        my $wrapper = sub {
+            pushHandler($type);
+            my $res = $inner_app->(@_);
+            popHandler();
+            return $res;
+        };
+        $obj->app($wrapper);
+    }
     count(1);
-    $handlerOR{$type} = \@Lemonldap::NG::Handler::Main::_onReload;
+
+    # Save the initialized handler data for future requests
+    $handlerOR{$type}   = [@Lemonldap::NG::Handler::Main::_onReload];
+    $handlerTSHV{$type} = $Lemonldap::NG::Handler::Main::_tshv;
+    pop @currenthandler;
     return $obj;
 }
 
-=head4 register
+=head4 withHandler
 
-Switch to a registered instance
+This method lets you run handler methods (such as conf reload)
+inside a give context
 
 =cut
 
-sub switch {
+sub withHandler {
+    my ( $type, $sub ) = @_;
+    pushHandler($type);
+    $sub->();
+    popHandler();
+}
+
+sub pushHandler {
     my $type = shift;
-    return [] unless $handlerOR{$type};
-    note( '==> Switching to ' . uc($type) . ' <==' );
+
+    # Save the current state of the previous handler, this is needed
+    # for tests in which portal initialization requires a HTTP request to
+    # another portal
+    if (@currenthandler) {
+        my $type = $currenthandler[-1];
+        note( '==> Saving handler  ' . uc($type) . ' <==' );
+        $handlerOR{$type}   = [@Lemonldap::NG::Handler::Main::_onReload];
+        $handlerTSHV{$type} = $Lemonldap::NG::Handler::Main::_tshv;
+    }
+    note( '==> Pushing ' . uc($type) . ' <==' );
+    push @currenthandler, $type;
     @Lemonldap::NG::Handler::Main::_onReload = @{
         $handlerOR{$type};
     };
+
+    $Lemonldap::NG::Handler::Main::_tshv = $handlerTSHV{$type};
+}
+
+sub popHandler {
+    my $type = pop @currenthandler;
+    note( '==> Popping ' . uc($type) . ' <==' );
+
+    # Restore previous handler context
+    if (@currenthandler) {
+        my $type = $currenthandler[-1];
+        return [] unless $handlerOR{$type};
+        note( '==> Restoring ' . uc($type) . ' <==' );
+        @Lemonldap::NG::Handler::Main::_onReload = @{
+            $handlerOR{$type};
+        };
+
+        $Lemonldap::NG::Handler::Main::_tshv = $handlerTSHV{$type};
+    }
+}
+
+# Not needed anymore, all the work is done by "register"
+sub switch {
+    note shortmess( 'Manual switching is deprecated,'
+          . ' you can remove it from your tests' );
 }
 
 =head4 encodeUrl( $url );
@@ -786,6 +893,7 @@ our $defaultIni = {
     },
     logLevel              => 'error',
     cookieName            => 'lemonldap',
+    languages             => 'en, fr',
     domain                => 'example.com',
     templateDir           => $templateDir,
     staticPrefix          => '/static',
@@ -869,9 +977,11 @@ has ini => (
         }
         if ( $ENV{DEBUG} ) {
             $ini->{logLevel} = 'debug';
+            $ini->{logger}   = "t::TestStdLogger";
         }
         if ( $ENV{LLNGLOGLEVEL} ) {
             $ini->{logLevel} = $ENV{LLNGLOGLEVEL};
+            $ini->{logger}   = "t::TestStdLogger";
         }
         $self->{ini} = $ini;
         main::ok( $self->{p} = $self->class->new(), 'Portal object' );
@@ -1042,7 +1152,7 @@ sub _get {
 
     my $res = $self->app->( {
             'HTTP_ACCEPT'          => $args{accept} // $self->accept,
-            'HTTP_ACCEPT_LANGUAGE' => 'fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3',
+            'HTTP_ACCEPT_LANGUAGE' => 'en-US,fr-FR;q=0.7,fr;q=0.3',
             'HTTP_CACHE_CONTROL'   => 'max-age=0',
             ( $args{cookie} ? ( HTTP_COOKIE => $args{cookie} ) : () ),
             'HTTP_HOST' => ( $args{host} ? $args{host} : 'auth.example.com' ),
@@ -1108,7 +1218,7 @@ sub _post {
       unless ( ref($body) and $body->can('read') );
     my $res = $self->app->( {
             'HTTP_ACCEPT'          => $args{accept} // $self->accept,
-            'HTTP_ACCEPT_LANGUAGE' => 'fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3',
+            'HTTP_ACCEPT_LANGUAGE' => 'en-US,fr-FR;q=0.7,fr;q=0.3',
             'HTTP_CACHE_CONTROL'   => 'max-age=0',
             ( $args{cookie} ? ( HTTP_COOKIE => $args{cookie} ) : () ),
             'HTTP_HOST' => ( $args{host} ? $args{host} : 'auth.example.com' ),
@@ -1179,4 +1289,17 @@ sub _put {
     return $self->_post( $path, $body, %args );
 }
 
+sub getHistory {
+    my ( $self, $uid, $only_type ) = @_;
+    my $psession = $self->p->getPersistentSession($uid);
+    my @entries;
+    my $history = $psession->data->{_loginHistory} || {};
+    for my $type ( keys %$history ) {
+        if ( !$only_type or $type eq "${only_type}Login" ) {
+            push @entries, @{ $history->{$type} || [] };
+        }
+    }
+    return @entries;
+}
+
 1;
